        NAME    STR300
;       created 03.04.93
;       Kleines Monitorprogramm, dass Ports lesen und setzen sowie das
;       interne RAM lesen und setzen kann.

version         equ             400             ; Version 4.00
; Version 4.0 wurde auf den AT89C2051 adaptiert


;       Fehlernummern
noerror         equ             0
e_cmdlen        equ             1               ; Befehlszeile zu lang
e_nocmd         equ             2               ; leere Zeile
e_unknowncmd    equ             3               ; unbekannter Befehl
e_hexzahl       equ             4               ; fehlerhafte Hexzahl
e_port          equ             5               ; Fehler in Portbefehl
e_ram           equ             6               ; Fehler in RAM-Befehl
e_reset         equ             7               ; Fehler oder Nein in Reset-Befehl
e_goto          equ             9               ; Fehler bei Goto-Befehl
e_goto_ndef     equ             0ah             ; Fehler: kein default gesetzt
e_error         equ             0dh             ; Fehler bei Error-Befehl

; Fehlernummern 20h bis 2fh fuer I2C-Routinen reserviert


;       Konstanten
maxcmdlen       equ             010h            ; max Zeilenlaenge
eocmdchar       equ             0dh             ; Zeichen bedeutet Ende von Zeile

;       Register-Baenke
BANK_MASK       equ             11100111b
BANK0           equ             00000000b
BANK1           equ             00001000b
BANK2           equ             00010000b
BANK3           equ             00011000b

;       BANK0 dient als allgemeine Befehlsinterpreterbank
cmd_bank        equ             BANK0           
i2cpoint        equ             00h             ; Zeiger fuer i2c-Routinen
w_i2cpoint      equ             R0              ; wiederhergestellt werden.
cmdpoint        equ             01h             ; Zeiger in Befehlszeile
w_cmdpoint      equ             R1 
cmdlen          equ             02h             ; Zeilenlaenge
w_cmdlen        equ             R2
cmdreg          equ             03h             ; allg. Register, braucht nicht
w_cmdreg        equ             R3              ; wiederhergstellt werden
misc_reg        equ             04h
w_misc_reg      equ             R4

;       BANK1 dient als Bank fuer binhex und die put-Routinen
put_bank        equ             BANK1
datpoint        equ             08h             ; Zeiger auf Daten
w_datpoint      equ             R0
datcount        equ             0ah             ; Zaehler fuer Daten
w_datcount      equ             R2              
datrec          equ             0bh             ; Datenregister zur
w_datrec        equ             R3              ;  Zwischenspeicherung


;       Datenregister
recpoint        equ             020h            ; Zeiger auf Empfangspuffer
reccount        equ             021h            ; Zaehler empfangene Zeichen
recok           equ             reccount.7      ; Flag fuer vollstaendige Zeile
errornr         equ             022h

recpuf          equ             030h            ; Empfangspuffer
cmdpuf          equ             040h            ; Puffer fuer Befehl


;       Stack  
STACK           equ             04fh            ; Registers 50..7F (24 calls)




CSEG            AT 00000h                

;       Resetvector und Interruptvectoren
reset_vector:   ajmp            start           ; Einsprung nach Reset

                org             003h
                ajmp            exint0

                org             00bh
                ajmp            t0int

                org             013h
                ajmp            exint1

                org             01bh
                ajmp            t1int

                org             023h
                ajmp            serint


;       dummy-int-routine beantwortet alle ints, die nicht gebraucht werden.
;       wenn auf einen der folgenden Interrupts reagiert werden soll, so mu
;       das Label aus der Liste entfernt und bei Ints, die per Software
;       zurueckgesetzt werden (adint, t2int), das entsprechende Bit in der
;       Maske gesetzt(!)  werden.
t0int:
t1int:
exint0:
exint1:
                        reti

;       die serielle Schnittstelle arbeitet mit den Bits RI und TI im Reg 
;       SCON. Wenn die serielle Schnittstelle per Interrupt bedient werden
;       soll, so mu die Routine beide Ereignisse (RI und TI) bearbeiten.
serint:                 push    PSW
                        push    ACC
                jnb             RI, si_nochar           ; weiter wenn kein Zeichen da
                        mov     PSW, #cmd_bank
                        mov     A, SBUF                 ; Zeichen holen
                        clr     RI                      ; Int zuruecksetzen
                acall           asciiup                 ; Grossbuchstaben
                cjne            A, #020h, si_w0         ; SPACE und LF herausfiltern
                sjmp            si_nochar
si_w0:          cjne            A, #00ah, si_w1
                sjmp            si_nochar
si_w1:                  push    cmdpoint                ; in INT-Routine ALLES retten        
                        mov     w_cmdpoint, recpoint
                        mov     @w_cmdpoint, A          ; Zeichen INDIRECT schreiben
                        pop     cmdpoint
                        inc     recpoint                ; Zeiger und Zaehler erhoehen
                        inc     reccount
                cjne            A, #eocmdchar, si_noteol
                        setb    recok                   ; Zeile zu Ende
si_noteol:              mov     A, reccount             ; Zeilenlaenge testen
                        anl     A, #01111111b           ; recok maskieren
                cjne            A, #maxcmdlen, si_nochar
                        mov     A, #e_cmdlen
                acall           errline                 ; Zeile zu lang:
                        mov     recpoint, #recpuf       ;  Fehlerzeile ausgeben
                        mov     reccount, #000h         ;  Puffer loeschen
si_nochar:              pop     ACC                     ; Register wiederherstellen
                        pop     PSW                     ;  und beenden
                reti
             

;       Texte
questreset:     db      'Neustart des Systems ausfuehren? ', 0


;       asciiup wandelt einen Kleinbuchstaben in A in einen Grossbuchstaben
;       um. Die Routine ist interrupt-geeignet. Andere Zeichen werden nicht
;       veraendert.
asciiup:        cjne            A, #123, au_w1
au_w1:          jnc             au_exit
                cjne            A, #97, au_w2
au_w2:          jc              au_exit
                        subb    A, #32
au_exit:        ret



;       hexbin wandelt eine 2-stellige hexadezimal Zahl in eine 8 Bit
;       Binaerzahl um. Die Hexzahl wird in A(low nibble) und B(high nibble)
;       uebergeben, das Ergebnis in A zurueckgegeben.
;       CARRY wird gesetzt, wenn die Hexzahl ein ungueltiges Zeichen enthaelt
;       Fehlernr: e_hexzahl             
hexbin:                 clr     C
                        subb    A, #'0'                 ; wandelt '0'..'9' um
                jc              hb_error                ; CY: kein Hexzeichen
                cjne            A, #010h, hb_w1         ; >0fh: 'A'..'F'
hb_w1:          jc              hb_w2
                        subb    A, #07h                 ; wandelt 'A'..'F' um
hb_w2:          cjne            A, #010h, hb_w3         ; >0fh: kein Hexzeichen
hb_w3:          jnc             hb_error
                        xch     A, B                    ; High Nibble wandlen
                        clr     C                       ; the same procedure
                        subb    A, #'0'                 ;  as every nibble
                jc              hb_error
                cjne            A, #010h, hb_w4
hb_w4:          jc              hb_w5
                        subb    A, #07h
hb_w5:          cjne            A, #010h, hb_w6
hb_w6:          jnc             hb_error
                        swap    A                       ; high nibble nach A.3-A.7
                        orl     A, B                    ; mit low nibble vereinigen
                        clr     C                       ; kein Fehler, kein CY
                sjmp            hb_exit
hb_error:               setb    C                       ; Fehler: CY und
                        mov     A, #e_hexzahl           ; FehlerNr in A
hb_exit:        ret

; read2par liesst zwei Werte aus dem Befehlspuffer und ruft hexbin auf.
read2par:               mov     B, @w_cmdpoint
                        inc     w_cmdpoint
                        mov     A, @w_cmdpoint
                ajmp            hexbin


;       binhex wandelt eine Binaerzahl mit bis zu 32 Bit in eine Hexzahl um.
;       In A wird die Laenge der Binaerzahl in BYTE(!) uebergeben, die
;       Binaerzahl wird in cmdpuf (Registerbank 3) uebergeben.
;       Das Ergebnis wird in cmdpuf zurueckgegeben.
;       Die Binaerzahl muss mit dem niegrig-wertigsten Byte zuerst stehen,
;       das Ergebnis steht mit dem hoechsten nibble zuerst(!)
;       Die Routine veraendert die Register datpoint, datcount und datrec.
binhex:                 push    B
                        push    PSW                     ; put_bank setzen
                        mov     PSW, #put_bank
                        mov     w_datcount, A
                        mov     w_datpoint, #cmdpuf
bh_loop1:               mov     B, @w_datpoint          ; Binaerzahl auf den
                        push    B                       ;  Stack
                        inc     w_datpoint
                djnz            ACC, bh_loop1
                        mov     B, w_datcount
                        mov     w_datpoint, #cmdpuf
bh_loop2:               pop     ACC                     ; Binaerbyte holen
                        mov     w_datrec, #0fh
                        anl     datrec, A               ; lower nibble retten
                        anl     A, #0f0h                ; lower nibble loeschen
                        swap    A
                        add     A, #'0'                 ; ergibt '0'...
                cjne            A, #':', bh_w1
bh_w1:          jc              bh_2nibble
                        add     A, #07h                 ; ergibt 'A'...'F'
bh_2nibble:             mov     @w_datpoint, A          ; Hex-Nibble speichern              
                        inc     w_datpoint              
                        mov     A, w_datrec             ; lower nibble laden
                        add     A, #'0'                 ; ergibt '0'...
                cjne            A, #':', bh_w2
bh_w2:          jc              bh_nextbyte
                        add     A, #07h                 ; ergibt 'A'...'F'
bh_nextbyte:            mov     @w_datpoint, A          ; 2. Hexnibble speichern              
                        inc     w_datpoint
                djnz            B, bh_loop2
                        mov     A, w_datcount           ; Anzahl Binaerbyte
                        rl      A                       ;  *2 = Anzahl Zeichen
                        pop     PSW                     
                        pop     B
                ret

;       binbcd wandelt eine binaerzahl in eine zweistellige BCD-Zahl um
binbcd:                 push    B
                        mov     B, #10
                        div     AB
                        anl     A, #00fh
                        swap    A
                        orl     A, B
                        pop     B
                ret



;       putbyte sendet ein Zeichen in A ueber die serielle Schnittstelle
;       TI muss bei der Initialisierung GESETZT werden
;       TI wird hardwaremaessig gesetzt, wenn ein Zeichen vollstaendig gesendet
;       ist. TI muss vor dem Schreiben in SBUF geloescht werden.
putbyte:                push    PSW                     ; Flags retten
                        push    ie
                        clr     ie.4                  ; serint abschalten
                        mov     SBUF, A                 ; Zeichen senden
pb_wait:        jnb             TI, pb_wait             ; warten bis gesendet
                        clr     TI                      ; Sendeflag loeschen
                        pop     ie                    ; serint wiederherstellen
                        pop     PSW
                ret

;       errline sendet "?<CR>", die Fehlernr des Fehlers wird in A uebergeben
errline:                push    ACC
                        mov     errornr, A              ; Fehlernr ablegen
                        mov     A, #'?'
                acall           putbyte                 ; Fehlerzeile ausgeben
                acall           putcrlf        
                        pop     ACC
                ret


;       prompt sendet '>' als Zeichen, dass MON8051 empfangsbereit ist.
prompt:                 push    ACC
                        mov     A, #'>'
                acall           putbyte
                        pop     ACC
                ret

;       putcrlf sendet 0dh, 0ah
putcrlf:                push    ACC
                        mov     A, #0dh
                acall           putbyte
                        mov     A, #0ah
                acall           putbyte
                        pop     ACC
                ret


;       putstring sendet einen mit 0 terminierten String, die Addresse mu
;       in DPTR uebergeben werden.
putstring:              push    ACC
                        push    B               
                        push    DPL
                        push    DPH
                        mov     B, #000h                ; B haelt den Index
ps_loop:                mov     A, B                    ; Index nach A
                        movc    A, @A+DPTR              ; Zeichen laden
                jz              ps_exit                 ; 0: Stringende
                        inc     B                       ; Index erhoehen
                acall           putbyte                 ; Zeichen ausgeben
                sjmp            ps_loop                 ; naechstes Zeichen
ps_exit:                pop     DPH                     ; Regs wiederherstellen
                        pop     DPL
                        pop     B
                        pop     ACC                     ;  und fertig.
                ret

;       putdat sendet eine Zeichenkette in cmdpuf mit der Laenge in A
;       veraendert die Register datpoint und datcount.
putdat:                 push    PSW                     ; put_bank setzen
                        push    ACC
                        push    ACC
                        mov     A, PSW
                        anl     A, #BANK_MASK
                        orl     A, #put_bank
                        mov     PSW, A
                        pop     datcount                ; Laenge Zeichenkette
                        mov     w_datpoint, #cmdpuf     ; Zeiger Zeichenkette
pd_loop1:               mov     A, @w_datpoint          ; Zeichen laden
                acall           putbyte                 ;  ausgeben    
                        inc     w_datpoint              ; Zeiger erhoehen
                djnz            w_datcount, pd_loop1    ; wiederholen
                        pop     ACC                     ; Laenge Zeichenkette
                        pop     PSW                     ; Flags und Bank
                ret

                        


;       getchar wartet auf ein Zeichen und liefert das Zeichen in A zurueck
;       Diese Routine arbeitet OHNE Interrupts!
getchar:                push    PSW
                        push    ie
                        clr     ie.4                  ; serint sperren
gc_loop:        jnb             RI, gc_loop             ; auf Zeichen warten
                        mov     A, SBUF                 ; Zeichen holen
                        clr     RI                      ; Empfangsbit loeschen
                        pop     ie                    ; serint wiederherstellen
                        pop     PSW
                ret



;       start: hier faengt die Ausfuehrung nach einem RESET an
start:  ; Variabeln initialisieren
                        mov     errornr, #noerror
                        mov     ie, #000h             ; alle Ints sperren
                        mov     sp, #STACK
                        mov     A, #0ffh              ; Ports auf definierten
                        mov     P1, A 
                        mov     P3, A
                        mov     TCON, #00000101b      ; ext. Ints triggern auf Flanke  

          ; cmd_bank setzen
                        mov     A, PSW
                        anl     A, #BANK_MASK
                        orl     A, #cmd_bank
                        mov     PSW, A
        
        ; serielle Schnittstellen initialisieren
                        mov     recpoint, #recpuf
                        mov     reccount, #0
                        mov     SCON, #01010000b
                        mov     PCON, #10000000b        ; SMOD=1
                        mov     TMOD, #00100000b
                        orl     TCON, #01000000b
                        mov     TH1, #0f3h
                        clr     TI                      ; "gesendet"-Flag
                        clr     RI                      ; "empfangen"-Flag

        ; Interrupt der seriellen Schnittstelle freigeben
                        orl     ie, #090h             ; EAL und ES setzen
                ajmp            cmdloop

;       Hautpschleife
;       ruft immerwaehrend proccmd auf, welches auf eine Befehlszeile wartet
;       und diese dann ausfhrt.

cmdloop:        acall           proccmd
                sjmp            cmdloop


;       Befehlsinterpreter
;       wartet solange, bis neuer Befehl da (recok gesetzt)
;       holt Befehl, loescht Empfangspuffer
;       bearbeitet 1. Zeichen des Befehls
proccmd:        acall           prompt

; Waehrend auf den naechsten Befehl gewartet wird, ist der slavemodus aktiv.
; Dazu muss i2c_bank als Registerbank gesetzt werden und alle Register
; koennen von der Interruptroutine veraendert werden.
pcloop0:               
                jnb             recok, pcloop0

                        clr     ie.4                    ; serint sperren
                        clr     recok                   ; Befehl da Flag loeschen
                        mov     w_cmdpoint, #recpuf     ; Zeiger auf Eingangspuffer
                        mov     A, #cmdpuf              ; Zeiger auf Commandpuffer
                        mov     w_cmdlen, reccount      ; Befehlslaenge
                        mov     B, w_cmdlen
pcloop1:                mov     misc_reg, @w_cmdpoint   ; Zeichen nach misc_reg
                        inc     w_cmdpoint              ; Zeiger Eingangspuffer erhoehen
                        xch     A, w_cmdpoint           ; Zeiger auf Commandpuffer
                        mov     @w_cmdpoint, misc_reg   ; Zeichen in Commandpuffer
                        inc     w_cmdpoint              ; zeiger Commandpuffer erhoehen
                        xch     A, w_cmdpoint           ; Zeiger auf Eingangspuffer
                djnz            B, pcloop1
                        mov     recpoint, #recpuf       ; Eingangspuffer freigeben
                        mov     reccount, #0
                        setb    ie.4                    ; serint wieder freigeben
                        mov     w_cmdpoint, #cmdpuf

;       Befehlszeile ist jetzt in cmdpuf.
;       1. Zeichen auswerten und
;         - zur weiteren Auswertung und Ausfuehrung an UnterPRG verzweigen
;         - leere Zeile: Fehlerzeichen ausgeben, beenden
;         - unbekannter Befehl: Fehlerzeichen ausgeben, beenden
                        mov     A, @w_cmdpoint
                        inc     w_cmdpoint
                        dec     w_cmdlen
                cjne            A, #eocmdchar, pc_w1
                        mov     A, #e_nocmd
                ajmp            errline
pc_w1:          cjne            A, #'P', pc_w2
                ajmp            procport
pc_w2:          cjne            A, #'D', pc_w4
                ajmp            procram
pc_w4:          cjne            A, #'E', pc_w5
                ajmp            procerror
pc_w5:          cjne            A, #'G', pc_w6
                ajmp            procgoto
pc_w6:          cjne            A, #'R', pc_w99
                ajmp            procreset

        ; alle bekannten Befehle abgefragt, jetzt "unknown command"
pc_w99:                 mov     A, #e_unknowncmd
                ajmp            errline
;       ende von proccmd



;       procport bearbeitet die Portbefehle
;       Es erwartet von proccmd einen Zeiger auf das naechste Zeichen in der
;       Befehlszeile.
;       Folgende Befehle sind definiert:
;               <CR>    Inhalt der Ports 1, 3
;               n<CR>   Inhalt des Ports n
;               nxx<CR> Port n auf xx setzen
;       Zusaetzliche Zeichen in der Befehlszeile fuer zur Fehlernr e_port
procport:               mov     A,@w_cmdpoint
                        inc     w_cmdpoint
                cjne            A, #eocmdchar, pp_getn   ; kein n: allout
                ajmp            pp_allout
pp_getn:                mov     B, #'0'                 ; n hat kein high nibble
                acall           hexbin                  ; n bestimmen
                jnc             pp_w1
                ajmp            errline                 ; Fehlerzeile ausgeben
pp_w1:          cjne            w_cmdlen, #02h, pp_w9   ; kein xx: outn
                sjmp            pp_outn
pp_w9:          cjne            w_cmdlen, #04h, pp_error
                        mov     w_cmdreg, A             ; n retten
                acall           read2par
                jnc             pp_w3                   ; Fehler wenn keine hexzahl
                ajmp            errline
pp_w3:          cjne            w_cmdreg, #01, pp_w5
                        mov     P1, A
                sjmp            pp_exit
pp_w5:          cjne            w_cmdreg, #03, pp_error
                        mov     P3, A
pp_exit:        ret
pp_error:               mov     A, #e_port              ; ungueltige Port nr
                ajmp            errline


pp_outn:        cjne            A, #01, po_w11
                        mov     cmdpuf, P1
                sjmp            po_w2
po_w11:         cjne            A, #03, pp_error
                        mov     cmdpuf, P3
po_w2:                  mov     A, #01
                acall           binhex
                acall           putdat
                ajmp            putcrlf

                        
pp_allout:              mov     A, #01h
                acall           pp_outn
                        mov     A, #03h
                sjmp            pp_outn


;       procram liesst aus und schreibt in einzelne Datenspeicherzellen aus
;       dem INTERNEN RAM des Prozessor
;       Es erwartet von proccmd einen Zeiger auf das naechste Zeichen in der
;       Befehlszeile.
;       Folgende Befehle sind definiert:
;               xx<CR>          Speicherstelle xx lesen
;               xxyy<CR>        yy in Speicherstelle schreiben.
procram:        cjne            w_cmdlen, #3, pr_w0
pr_w0:          jc              pr_error                ; < 5: Fehler
                        mov     B, @w_cmdpoint          ; High Byte laden
                        inc     w_cmdpoint
                        mov     A, @w_cmdpoint
                        inc     w_cmdpoint
                acall           hexbin
                jnc             pr_w1
                ajmp            errline                 ; Fehler: keine Hexzahl
pr_w1:                  mov     w_misc_reg, A           ; Adresse retten
                cjne            w_cmdlen, #3, pr_write
                        mov     w_cmdpoint, misc_reg
                        mov     A, @w_cmdpoint
                        mov     cmdpuf, A
                        mov     A, #01h
                acall           binhex
                acall           putdat
                ajmp            putcrlf

pr_write:       cjne            w_cmdlen, #5, pr_error
                        mov     B, @w_cmdpoint
                        inc     w_cmdpoint
                        mov     A, @w_cmdpoint
                acall           hexbin
                jnc             pr_w3
                ajmp            errline
pr_w3:                  mov     w_cmdpoint, misc_reg
                        mov     @w_cmdpoint, A
                ret

pr_error:               mov     A, #e_ram
                ajmp            errline


;       procreset fragt ob wirlich neu gestartet werden soll. Folgende 
;       Befehle ind definiert
;               R<CR>   Abfragen: wirklich j/n
;               RJ<CR>  Reset ohne Abfrage ausfuehren
;               RX<CR>  jedes andere Zeichen: kein Reset
procreset:      cjne            w_cmdlen, #02, pr_frage
                cjne            @w_cmdpoint, #'J', pr_noreset
                ajmp            reset_vector
pr_noreset:             mov     A, #e_reset
                ajmp            errline

pr_frage:               mov     DPTR, #questreset
                acall           putstring               ; Fragetext ausgeben
                        clr     RI
                acall           getchar                 ; Anwortzeichen holen
                acall           asciiup 
                cjne            A, #'J', pr_noreset     ; nur bei 'J' reset
                ajmp            reset_vector



;       procgoto ermglicht es eine beliebige Routine anzuspringen
;       Format:
;               xxxx<CR>        bewirkt: acall xxxx
;       Wenn beim Rueckpsrung Carry gesetzt ist, wird eine Fehlermeldung
;       ausgegeben und <A> als Fehlernummer gespeichert
procgoto:       cjne            w_cmdlen, #5, pg_error1
                        mov     B, @w_cmdpoint          ; Zieladdresse holen
                        inc     w_cmdpoint              ; Konvertierung
                        mov     A, @w_cmdpoint          ;  hex -> bin
                        inc     w_cmdpoint
                acall           hexbin
                jc              pg_error2               ; keine hexzahl: Fehler
                        mov     DPH, A                  ; highbyte zuerst
                        mov     B, @w_cmdpoint
                        inc     w_cmdpoint
                        mov     A, @w_cmdpoint
                        inc     w_cmdpoint
                acall           hexbin                  ; lowbyte konvertieren
                jc              pg_error2
                        mov     DPL, A     
                        clr     A
                acall           pg_w3
                jnc             pg_w2
                ajmp            errline
pg_w2:          ret

pg_w3:          jmp             @A+DPTR                 ; Ausfuehren !!!

pg_error1:              mov     A, #e_goto
pg_error2:      ajmp            errline



;       procerror gibt die Fehlernr aus. Die Fehlernr wird anschiessend
;       auf 0 (noerror) zurueckgesetzt
;       Folgende Befehle sind definiert:
;               <CR>    Fehlernr ausgeben
procerror:              mov     cmdpuf, errornr         ; Fehlernr in puffer
                        mov     A, #01                  ; Laenge Binaerzahl
                        mov     errornr, #00            ; Fehlernr loeschen
                acall           binhex                  ; Hexzahl machen,
                acall           putdat                  ;  ausgeben und
                ajmp            putcrlf                 ;  neue Zeile

 
        END





